--
-- Cargos
--

Cargos = {};
Cargos.FORAGE  = 0;
Cargos.SILAGE  = 1;

function Cargos.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function Cargos:load(xmlFile)
    self.ForagefillTypes = {};
    self.ForagefillTypes[FruitUtil.FRUITTYPE_UNKNOWN] = true;

    local ForagefruitTypes = getXMLString(xmlFile, "vehicle.ForagefillTypes#fruitTypes");
    if ForagefruitTypes ~= nil then
        local Foragetypes = Utils.splitString(" ", ForagefruitTypes);
        for k,v in pairs(Foragetypes) do
            local desc = FruitUtil.fruitTypes[v];
            if desc ~= nil then
                self.ForagefillTypes[desc.index] = true;
            end;
        end;
    end;
    
    self.SilagefillTypes = {};
    self.SilagefillTypes[FruitUtil.FRUITTYPE_UNKNOWN] = true;

    local SilagefruitTypes = getXMLString(xmlFile, "vehicle.SilagefillTypes#fruitTypes");
    if SilagefruitTypes ~= nil then
        local Silagetypes = Utils.splitString(" ", SilagefruitTypes);
        for k,v in pairs(Silagetypes) do
            local desc = FruitUtil.fruitTypes[v];
            if desc ~= nil then
                self.SilagefillTypes[desc.index] = true;
            end;
        end;
    end;
    

    local forageWgnSound = getXMLString(xmlFile, "vehicle.forageWgnSound#file");
    if forageWgnSound ~= nil and forageWgnSound ~= "" then
        forageWgnSound = Utils.getFilename(forageWgnSound, self.baseDirectory);
        self.forageWgnSound = createSample("forageWgnSound");
        self.forageWgnSoundEnabled = false;
        loadSample(self.forageWgnSound, forageWgnSound, false);
        self.forageWgnSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.forageWgnSound#pitchOffset"), 1);
        self.forageWgnSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.forageWgnSound#volume"), 1.0);
    end;
	
		local pickupNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.pickup#index"));
    if pickupNode ~= nil then
        self.pickup = {};
        self.pickup.node = pickupNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.pickup#minRot"));
        self.pickup.minRot = {};
        self.pickup.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.pickup.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.pickup.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.pickup#maxRot"));
        self.pickup.maxRot = {};
        self.pickup.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.pickup.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.pickup.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        self.pickup.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.pickup#rotTime"), 2)*1000;
        self.pickup.touchRotLimit = Utils.degToRad(Utils.getNoNil(getXMLString(xmlFile, "vehicle.pickup#touchRotLimit"), 10));
    end;
	
    self.wholepickup= Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.wholepickup#index"));
    self.pickupcover= Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.wholepickup#cover"));
    
    self.drum1= Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.drum#index1"));
    self.drumRotationScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.drum#rotationScale"), 1);

    self.crusher= Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.crusher#index1"));
    self.crusherRotationScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.crusher#rotationScale"), 1);

    self.spreadersCount = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.spreaders#count"), 0);
    if self.spreadersCount > 0 then
      self.spreadersSpeed = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.spreaders#speed"), 0.01);
      self.spreaders = {};
      for i = 1, self.spreadersCount do
        local sid = string.format("vehicle.spreaders.spreader%d#index", i);
        local spread = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, sid));
        table.insert(self.spreaders,i,spread);
      end;
    end;

    local grainTipAnimCurve = AnimCurve:new(linearInterpolator4);
    local keyI = 0;
    while true do
      local key = string.format("vehicle.grainPlane.tipkey(%d)", keyI);
      local t = getXMLFloat(xmlFile, key.."#time");
      local yValue = getXMLFloat(xmlFile, key.."#y");
      local scaleX,scaleY,scaleZ = Utils.getVectorFromString(getXMLString(xmlFile, key.."#scale"));
      if y == nil or scaleX == nil or scaleY == nil or scaleZ == nil then
        break;
      end;
      grainTipAnimCurve:addKeyframe({x=scaleX, y=scaleY, z=scaleZ, w=yValue, time = t});
      keyI = keyI +1;
    end;
    if keyI > 0 then
      self.grainTipAnimCurve = grainTipAnimCurve;
    end;
    
    self.reverseLightObject = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.reverselights#index"));
    setVisibility(self.reverseLightObject, false);
    self.reverseLightActive = false;

    self.brakeLightObject = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.brakelights#index"));
    setVisibility(self.brakeLightObject, false);
    self.brakeLightActive = false;


    self.fillScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillScale#value"), 1);
    self.mode = Cargos.FORAGE;
    self.fillTypes = self.ForagefillTypes;
    self.wasToFast = false;
    self.isTurnedOn = false;
	  self.currentFruitType = FruitUtil.FRUITTYPE_UNKNOWN;
	  self.lastFruitType = FruitUtil.FRUITTYPE_UNKNOWN;
	  self.pickupMax = false;
	  self.allowFillFromAir = false;
	  self.TipTriggerInRange = false;
    self.RaycastReferencePoint1 = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.raycastReferencePoint#index1"));
    self.RaycastReferencePoint2 = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.raycastReferencePoint#index2"));
    self.findTrailerRaycastCallback = Cargos.findTrailerRaycastCallback;	  
end;

function Cargos:delete()
    if self.forageWgnSound ~= nil then
        delete(self.forageWgnSound);
    end;
end;

function Cargos:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
    local cargosMode = getXMLString(xmlFile, key.."#cargosMode");
    if cargosMode ~= nil then
      if cargosMode == "forage" then
        self.isTurnedOn = false;
  	    self.pickupMax = false;
		    self.mode = Cargos.FORAGE;
		    self.fillTypes = self.ForagefillTypes;
        setVisibility(self.wholepickup, true); 
        setVisibility(self.pickupcover, false);
        self.allowFillFromAir = false;     
      end;
      if cargosMode == "silage" then
        self.isTurnedOn = false;
			  self.pickupMax = false;
			  self.mode = Cargos.SILAGE;
			  self.fillTypes = self.SilagefillTypes;
        setVisibility(self.wholepickup, false); 
        setVisibility(self.pickupcover, true);
        self.allowFillFromAir = true;     
      end;      
    end;
    return BaseMission.VEHICLE_LOAD_OK;
end;

function Cargos:getSaveAttributesAndNodes(nodeIdent)
    local cargosMode = "forage";
    if self.mode == Cargos.SILAGE then
      cargosMode = "silage";
    end;
    local attributes = 'cargosMode="'..cargosMode..'"';
    return attributes, nil;
end;


function Cargos:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Cargos:keyEvent(unicode, sym, modifier, isDown)
end;

function Cargos:update(dt)
  
	if self.spreadersCount > 0 then
    local v = true;
    if ((self.currentFillType == FruitUtil.FRUITTYPE_WHEAT) or (self.currentFillType == FruitUtil.FRUITTYPE_BARLEY)) then
      v = false;
    end;
    if (self.tipState == Trailer.TIPSTATE_CLOSING) or (((self.tipState == Trailer.TIPSTATE_OPENING) or (self.tipState == Trailer.TIPSTATE_OPEN)) and (self.fillLevel == 0)) then
      if ((self.lastFruitType == FruitUtil.FRUITTYPE_WHEAT) or (self.lastFruitType == FruitUtil.FRUITTYPE_BARLEY)) then
        v = false;
      end;
    end; 
	  for i=1, self.spreadersCount do
	    if self.spreaders[i] ~= nil then
	      setVisibility(self.spreaders[i], v);	      
	    end;
	  end;
	end; 
  
  if self.mode == Cargos.FORAGE then
    if self:getIsActiveForInput() then
      if (InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2)) and (self.fillLevel == 0) then
        self.isTurnedOn = false;
			  self.pickupMax = false;
			  self.mode = Cargos.SILAGE;
			  self.fillTypes = self.SilagefillTypes;
        setVisibility(self.wholepickup, false); 
        setVisibility(self.pickupcover, true);
        self.allowFillFromAir = true;
      end;

      if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
        self.isTurnedOn = not self.isTurnedOn;
			  self.pickupMax = not self.pickupMax;
      end;
      
    end;
	
	  if self.pickup ~= nil then
        local x, y, z = getRotation(self.pickup.node);
        local rot = {x,y,z};
        local newRot = Utils.getMovedLimitedValues(rot, self.pickup.maxRot, self.pickup.minRot, 3, self.pickup.rotTime, dt, not self.pickupMax);
        setRotation(self.pickup.node, unpack(newRot));
    end;
	
	  if self.drum1 ~= nil and self.isTurnedOn then
		  rotate(self.drum1, self.drumRotationScale * 100, 0, 0); 
	  end;

	  if self.crusher ~= nil and self.isTurnedOn then
		  rotate(self.crusher, self.crusherRotationScale * 100, 0, 0); 
	  end;
	  
    self.wasToFast = false;
    if self:getIsActive() then
        if self.isTurnedOn then
			      
            local toFast = self:doCheckSpeedLimit() and self.attacherVehicle.lastSpeed*3600 > 29;
            if not toFast then				
				      local area =0;
              for k, cuttingArea in pairs(self.cuttingAreas) do
                local x,y,z = getWorldTranslation(cuttingArea.start);
                local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
                local x2,y2,z2 = getWorldTranslation(cuttingArea.height);

                for fruitType,v in pairs(self.fillTypes) do
                  if fruitType ~= FruitUtil.FRUITTYPE_UNKNOWN then
							      if self.currentFruitType == fruitType or self.currentFruitType == FruitUtil.FRUITTYPE_UNKNOWN then
								      local area = Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, 0)*g_currentMission.windrowCutLongRatio;
								      area = area + Utils.updateFruitCutLongArea(fruitType, x, z, x1, z1, x2, z2, 0);
								      if area > 0 then
									      self.currentFruitType = fruitType;
								      end;
								      local pixelToQm = 2048 / 4096 * 2048 / 4096  / g_currentMission.maxFruitValue; -- 4096px are mapped to 2048m
								      local literPerQm = FruitUtil.fruitIndexToDesc[fruitType].literPerQm * (1 + 0.5 * (3 - g_currentMission.missionStats.difficulty));
								      local qm = area*pixelToQm;
								      local deltaLevel = qm*literPerQm * self.fillScale;
								      self:setFillLevel(self.fillLevel+deltaLevel, self.currentFruitType);
							      end;
                  end;
                end;
              end;	
				      if self.fillLevel == 0 then
					      self.currentFruitType = FruitUtil.FRUITTYPE_UNKNOWN;					      
				      end;			
            end;

            self.wasToFast = toFast;
			
            if not self.forageWgnSoundEnabled and self:getIsActiveForSound() then
                playSample(self.forageWgnSound, 0, self.forageWgnSoundVolume, 0);
                setSamplePitch(self.forageWgnSound, self.forageWgnSoundPitchOffset);
                self.forageWgnSoundEnabled = true;
            end;
            
            if self.fillLevel == self.capacity then
              self.isTurnedOn = false;
              self.pickupMax = false;
            end;
        end;

        if self.forageWgnSoundEnabled and not self.isTurnedOn then
            stopSample(self.forageWgnSound);
            self.forageWgnSoundEnabled = false;
        end;

    end;
  else
    if self:getIsActiveForInput() then
      if (InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2)) and (self.fillLevel == 0) then
        self.isTurnedOn = false;
  	    self.pickupMax = false;
		    self.mode = Cargos.FORAGE;
		    self.fillTypes = self.ForagefillTypes;
        setVisibility(self.wholepickup, true); 
        setVisibility(self.pickupcover, false);
        self.allowFillFromAir = false;
      end;  
    end; 
  end;
	if self.tipState == Trailer.TIPSTATE_OPENING or self.tipState == Trailer.TIPSTATE_OPEN then
	  if (self.fillLevel ~= 0) and (self.currentFillType ~= FruitUtil.FRUITTYPE_UNKNOWN) then
	    self.lastFruitType = self.currentFillType;
    end; 
    if self.spreadersCount > 0 then
	    for i=1, self.spreadersCount do
	      if self.spreaders[i] ~= nil then
	        rotate(self.spreaders[i],self.spreadersSpeed * 100,0,0);
	      end;
	    end;
	  end;
	end; 
	if self.tipState == Trailer.TIPSTATE_CLOSED and self.fillLevel==0 then
	  self.lastFruitType = self.currentFillType;
	end;

  if (self.tipState == Trailer.TIPSTATE_CLOSED) and (self.fillLevel>0) then  
    local x,y,z = getWorldTranslation(self.RaycastReferencePoint1);
    self.TipTriggerInRange = false;    
    raycastAll(x, y, z, 0, 0, -1, "findTrailerRaycastCallback", 10, self);
    raycastAll(x, y, z, 0, 0,  1, "findTrailerRaycastCallback", 10, self);
    local x,y,z = getWorldTranslation(self.RaycastReferencePoint2);    
    raycastAll(x, y, z, 0, 0,  1, "findTrailerRaycastCallback", 10, self);
    raycastAll(x, y, z, 0, 0, -1, "findTrailerRaycastCallback", 10, self);

    local fillTypeName = FruitUtil.fruitIndexToDesc[self.currentFillType].name;
    local grainPlane = self.grainPlanes[fillTypeName];
     
    local x, y, z = getTranslation(grainPlane);
    local trans = {x,y,z}; 
    local scaleX, scaleY, scaleZ , yTrans = self.grainTipAnimCurve:get(self.fillLevel/self.capacity);
    local sX, sY, sZ , yMax = self.grainAnimCurve:get(self.fillLevel/self.capacity);
    local newtrans = Utils.getMovedLimitedValues(trans,{x,yMax,z}, {x,yTrans,z}, 3, 4000, dt, self.TipTriggerInRange); 
    setTranslation(grainPlane, unpack(newtrans)); 
  end;

  -- Reverse lights
  if self.movingDirection < 0 then
    self.reverseLightActive = true;
  else
    self.reverseLightActive = false;
  end;
  setVisibility(self.reverseLightObject, self.reverseLightActive);
  
  -- Brake lights
  local input = 0;
  if self:getIsActiveForInput() then
      input = InputBinding.getAnalogInputAxis(InputBinding.AXIS_FORWARD);
      if InputBinding.isAxisZero(input) then
          input = InputBinding.getDigitalInputAxis(InputBinding.AXIS_FORWARD);
      end;
  end;
  
  if (self.movingDirection*self.lastSpeed*(-input)) < -0.001 then
    self.brakeLightActive = true;
  else
    self.brakeLightActive = false;
  end;
  setVisibility(self.brakeLightObject, self.brakeLightActive);
    
end;

function Cargos:draw()
  if self.mode == Cargos.FORAGE then
    if self.wasToFast then
        g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "2", InputBinding.getButtonKeyName(InputBinding.SPEED_LEVEL2)), 0.07+0.022, 0.019+0.029);
    end;
    if self.isTurnedOn then
        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_off_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
    else
        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_on_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
        if (self.fillLevel == 0) then 
          g_currentMission:addHelpButtonText(g_i18n:getText("cargos_switch_silage"), InputBinding.IMPLEMENT_EXTRA2);
        end;
    end;
  else
    if (self.fillLevel == 0) then
      g_currentMission:addHelpButtonText(g_i18n:getText("cargos_switch_forage"), InputBinding.IMPLEMENT_EXTRA2);
    end;
  end;
end;

function Cargos:setFillLevel(fillLevel, fillType)
    if not self:allowFillType(fillType, false) then
        return;
    end;

    self.currentFillType = fillType;
    self.fillLevel = fillLevel;
    if self.fillLevel > self.capacity then
        self.fillLevel = self.capacity;
    end;
    if self.fillLevel < 0 then
        self.fillLevel = 0;
        -- reset fill type as the trailer is empty
        Utils.setEmittingState(self.dischargeParticleSystems[self.currentFillType], false);
        self.currentFillType = FruitUtil.FRUITTYPE_UNKNOWN;
    end;
    if self.currentGrainPlane ~= nil then
        setVisibility(self.currentGrainPlane, false);
    end;
    if self.grainPlanes ~= nil and self.defaultGrainPlane ~= nil and fillType ~= FruitUtil.FRUITTYPE_UNKNOWN then
        local fillTypeName = FruitUtil.fruitIndexToDesc[fillType].name;
        local grainPlane = self.grainPlanes[fillTypeName];
        if grainPlane == nil then
            grainPlane = self.defaultGrainPlane;
        end;
        local yTranslation;
        if self.tipState == Trailer.TIPSTATE_OPENING or self.tipState == Trailer.TIPSTATE_OPEN or self.tipState == Trailer.TIPSTATE_CLOSING then
          if self.grainTipAnimCurve then
            local scaleX, scaleY, scaleZ , yTrans = self.grainTipAnimCurve:get(self.fillLevel/self.capacity);
            yTranslation = yTrans;
            setScale(grainPlane, scaleX, scaleY, scaleZ);
          else
            local m = (self.grainPlaneMaxY - self.grainPlaneMinY) / self.capacity;
            yTranslation = m*self.fillLevel + self.grainPlaneMinY;
          end;
        else 
          if self.grainAnimCurve then
            local scaleX, scaleY, scaleZ , yTrans = self.grainAnimCurve:get(self.fillLevel/self.capacity);
            yTranslation = yTrans;
            setScale(grainPlane, scaleX, scaleY, scaleZ);
          else
            local m = (self.grainPlaneMaxY - self.grainPlaneMinY) / self.capacity;
            yTranslation = m*self.fillLevel + self.grainPlaneMinY;
          end;
        end;
        local xPos, yPos, zPos = getTranslation(grainPlane);
        setTranslation(grainPlane, xPos, yTranslation, zPos);
        setVisibility(grainPlane, self.fillLevel > 0);
        self.currentGrainPlane = grainPlane;
    end;
end;

function Cargos:onDetach()
    if self.deactivateOnDetach then
        Cargos.onDeactivate(self);
    else
        Cargos.onDeactivateSounds(self);
    end;
end;

function Cargos:onLeave()
    if self.deactivateOnLeave then
        Cargos.onDeactivate(self);
    else
        Cargos.onDeactivateSounds(self);
    end;
end;

function Cargos:onDeactivate()
    if self.isTurnedOn then
      self.pickupMax = false;
    end;
    self.isTurnedOn = false;
    Cargos.onDeactivateSounds(self);
end;

function Cargos:onDeactivateSounds()
    if self.forageWgnSoundEnabled then
        stopSample(self.forageWgnSound);
        self.forageWgnSoundEnabled = false;
    end;
end;

function Cargos:findTrailerRaycastCallback(transformId, x, y, z, distance)
  if transformId ~= self.rootNode then
    local nalezeno = false;
    for i=1, table.getn(g_currentMission.tipTriggers) do
      nalezeno=nalezeno or (g_currentMission.tipTriggers[i].triggerId==transformId);    
    end;
    if nalezeno then
      self.TipTriggerInRange = true;
    end;
  end; 
  return nalezeno;
end;
